@@ -503,6 +503,21 @@ if(!bivariate)
       }
 }
 
+#################################################################################
+################ wrapped Gaussian RF                     #######################
+#################################################################################
+
+
+if(model==13)   ##  wrapped
+    {
+      if(!bivariate) {
+         corr=cr$corr*(1-as.numeric(nuisance['nugget']))
+         vv=sinh(as.numeric(nuisance['sill']))
+         corr=sinh(as.numeric(nuisance['sill'])*corr)/vv
+        }
+     if(bivariate){}
+    }
+
 #################################################################################
 ################ models defined on the positive real line #######################
 #################################################################################
